import json
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass

from loguru import logger

from src.core import api_request
from src.core.constant import VISUAL_LIST_URL
from src.core.util import get_curr_day, get_curr_month
from src.entity.account import Account
from src.entity.user import User, get_user_by_telegram_id


# 视图列表对象 对应界面上的图表
@dataclass
class VisualInfo:
    staticsDate: str
    agentId: int
    agentType: int
    agentCode: str
    agentName: str
    isNew: int
    firstDeposit: float
    deposit: float
    depositPromo: float
    draw: int
    promo: float
    promoDividend: float
    rebate: float
    adjust: float
    riskAdjust: float
    bets: float
    profit: float
    allBets: float
    firstCount: int
    countDeposit: int
    countDraw: int
    countBets: int
    createdAt: str
    updatedAt: str
    oldDeposit: float
    oldDepositCount: int
    newDeposit: float

    @staticmethod
    def from_dict(data):
        return VisualInfo(**data)


@dataclass
class VisualList:
    curData: list[VisualInfo]
    lastData: list[VisualInfo]

    @staticmethod
    def from_json(json_str):
        data = json.loads(json_str)
        cur_data = [VisualInfo.from_dict(item) for item in data['curData']]
        last_data = [VisualInfo.from_dict(item) for item in data['lastData']]
        return VisualList(curData=cur_data, lastData=last_data)

    @staticmethod
    def dict_to_visual_list(data):
        cur_data = [VisualInfo(**item) for item in data['curData']]
        last_data = [VisualInfo(**item) for item in data['lastData']]
        return VisualList(curData=cur_data, lastData=last_data)


def get_visual_list(account: Account, params: dict) -> VisualList:
    res = api_request.account_post(url=VISUAL_LIST_URL, account=account,
                                   params=params)
    return VisualList.dict_to_visual_list(res.data)


def get_curr_data(account: Account, params: dict) -> list[VisualInfo]:
    res = api_request.account_post(url=VISUAL_LIST_URL, account=account,
                                   params=params)
    return [VisualInfo(**item) for item in res.data['curData']]


async def async_get_visual_list(account: Account, params: dict) -> VisualList:
    res = await api_request.async_account_post(url=VISUAL_LIST_URL, account=account,
                                               params=params)
    return VisualList(**res.data)


async def async_get_curr_data(account: Account, params: dict) -> list[VisualInfo]:
    res = await api_request.async_account_post(url=VISUAL_LIST_URL, account=account,
                                               params=params)
    return [VisualInfo(**item) for item in res.data['curData']]


def get_statics(account, date=get_curr_day()) -> VisualInfo:
    params = {"monthDate": get_curr_month()}
    data = get_visual_list(account, params)
    # 合并列表并创建日期到数据的映射
    date_map = {item.staticsDate: item for item in data.curData + data.lastData}

    # 直接通过日期获取数据
    return date_map.get(date)


def count_by_user(user: User, date: str):
    accounts = user.accounts
    with ThreadPoolExecutor(max_workers=len(accounts)) as t:
        futures = [t.submit(get_statics, account, date) for account in accounts]
        return [future.result() for future in futures]


def text_count_by_user(user: User, date: str) -> str:
    visual_list = count_by_user(user, date)
    text = '\n\n'.join(
        # f'{result.agentName}\n注册：{result.isNew}\n首存：{result.firstCount}\n日活：{int(result.countBets)}\n流水：{int(result.bets)}'
        f'{result.agentName}\n注册：{result.isNew}\n首存：{result.firstCount}\n日活：{int(result.countBets)}'
        for result in visual_list
    )
    logger.info(f'Generated text: {text}')
    return text


def text_count_by_telegram_id(telegram_id: int) -> str:
    visual_list = count_by_user(get_user_by_telegram_id(telegram_id), get_curr_day())
    text = '\n\n'.join(
        f'{result.agentName}\n注册：{result.isNew}\n首存：{result.firstCount}\n日活：{int(result.countBets)}\n流水：{int(result.bets)}'
        for result in visual_list
    )
    logger.info(f'Generated text: {text}')
    return text